"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const airtable_utils_1 = require("./airtable_utils");
class DataProvider {
    constructor(base, filterParser) {
        this.filterParser = filterParser;
        this.base = base;
    }
    find(collectionName, filter, sort, skip, limit, projection) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const filterByFormula = this.filterToFilterByFormula(filter);
            const limitExpr = limit ? { maxRecords: limit } : {};
            const sortExpr = this.filterParser.orderBy(sort);
            const projectionExpr = this.filterParser.selectFieldsFor(projection);
            const result = yield this.query({ collectionName, filterByFormula, projection: projectionExpr, limitExpr, sortExpr, skip });
            return result.map(airtable_utils_1.minifyAndFixDates);
        });
    }
    count(collectionName, filter) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let count = 0;
            yield this.base(collectionName)
                .select(this.filterToFilterByFormula(filter))
                .eachPage((records, fetchNextPage) => {
                count += records.length;
                fetchNextPage();
            });
            return count;
        });
    }
    insert(collectionName, items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const createExpr = this.bulkCreateExpr(items);
            const inserted = yield this.base(collectionName)
                .create(createExpr);
            return inserted.length;
        });
    }
    update(collectionName, items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const updated = yield Promise.all(items.map((item) => tslib_1.__awaiter(this, void 0, void 0, function* () { return yield this.updateSingle(collectionName, item); })));
            return updated.length;
        });
    }
    delete(collectionName, itemIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const ids = yield Promise.all(itemIds.map((id) => tslib_1.__awaiter(this, void 0, void 0, function* () { return yield this.wixDataIdToAirtableId(collectionName, id); })));
            const deleted = yield this.base(collectionName)
                .destroy(ids);
            return deleted.length;
        });
    }
    truncate(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.base(collectionName)
                .select()
                .eachPage((records, fetchNextPage) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.base(collectionName)
                    .destroy(records.map((record) => record.id));
                fetchNextPage();
            }));
        });
    }
    query({ collectionName, filterByFormula, limitExpr, sortExpr, idsOnly, skip, projection }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resultsByPages = [];
            const limit = (limitExpr === null || limitExpr === void 0 ? void 0 : limitExpr.maxRecords) ? limitExpr : { maxRecords: airtable_utils_1.DEFAULT_MAX_RECORDS };
            limit.maxRecords += skip;
            const sort = airtable_utils_1.EmptySort;
            if (sortExpr && sortExpr.sort)
                Object.assign(sort, sortExpr);
            yield this.base(collectionName)
                .select(Object.assign(Object.assign(Object.assign({}, filterByFormula), limitExpr), sort))
                .eachPage((records, fetchNextPage) => {
                let recordsToReturn;
                if (idsOnly)
                    recordsToReturn = records.map((record) => record.id);
                else {
                    recordsToReturn = projection ? records.map((record) => {
                        return projection.reduce((pV, cV) => (Object.assign(Object.assign({}, pV), { [cV]: record.fields[cV] })), {});
                    }) : records.map((record) => record.fields);
                }
                resultsByPages.push(recordsToReturn);
                fetchNextPage();
            });
            return resultsByPages.flat().slice(skip); //TODO: find other solution for skip! at least don't load everything to memory.    
        });
    }
    filterToFilterByFormula(filter) {
        // @ts-ignore
        const { filterExpr } = this.filterParser.transform(filter);
        return { filterByFormula: filterExpr || '' };
    }
    wixDataIdToAirtableId(collectionName, _id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const record = yield this.query({ collectionName, filterByFormula: { filterByFormula: `_id = "${_id}" ` }, idsOnly: true });
            return record[0];
        });
    }
    updateSingle(collectionName, item) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const id = yield this.wixDataIdToAirtableId(collectionName, item._id);
            const updated = yield this.base(collectionName)
                .update(id, item);
            return updated.id;
        });
    }
    bulkCreateExpr(items) {
        return items.reduce((pV, cV) => {
            pV.push({ fields: Object.assign({}, cV) });
            return pV;
        }, []);
    }
}
exports.default = DataProvider;
//# sourceMappingURL=airtable_data_provider.js.map