"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const sql_schema_translator_1 = require("./sql_schema_translator");
const velo_external_db_commons_1 = require("@wix-velo/velo-external-db-commons");
const axios_1 = require("axios");
const velo_external_db_types_1 = require("@wix-velo/velo-external-db-types");
const velo_external_db_commons_2 = require("@wix-velo/velo-external-db-commons");
const { CollectionDoesNotExists, FieldAlreadyExists, FieldDoesNotExist } = velo_external_db_commons_2.errors;
class SchemaProvider {
    constructor(base, { apiKey, metaApiKey, baseUrl }) {
        this.base = base;
        this.baseId = base.getId();
        this.sqlSchemaTranslator = new sql_schema_translator_1.default();
        this.axios = axios_1.default.create({
            baseURL: baseUrl || 'https://api.airtable.com',
            headers: {
                Authorization: `Bearer ${apiKey}`,
                'X-Airtable-Client-Secret': metaApiKey
            }
        });
    }
    list() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const response = yield this.axios.get(`v0/meta/bases/${this.baseId}/tables`);
            const tables = this.extractTableData(response.data);
            return Object.entries(tables)
                .map(([collectionName, rs]) => ({
                id: collectionName,
                fields: rs.fields
            }));
        });
    }
    listHeaders() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const response = yield this.axios.get(`v0/meta/bases/${this.baseId}/tables`);
            return response.data.tables.map((rs) => rs.name);
        });
    }
    supportedOperations() {
        const { List, ListHeaders, Create, Drop, AddColumn, RemoveColumn, Describe, BulkDelete, Truncate, DeleteImmediately, UpdateImmediately } = velo_external_db_types_1.SchemaOperations;
        return [List, ListHeaders, Create, Drop, AddColumn, RemoveColumn, Describe, BulkDelete, Truncate, DeleteImmediately, UpdateImmediately];
    }
    create(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const systemColumnsAsAirTableColumns = velo_external_db_commons_1.SystemFields.map(field => this.sqlSchemaTranslator.wixColumnToAirtableColumn(field));
            yield this.axios.post(`v0/meta/bases/${this.baseId}/table`, { collectionName, columns: systemColumnsAsAirTableColumns });
        });
    }
    drop(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.axios.post(`v0/meta/bases/${this.baseId}/table/drop`, { collectionName });
        });
    }
    addColumn(collectionName, column) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, velo_external_db_commons_1.validateSystemFields)(column.name);
            const collection = yield this.describeCollection(collectionName);
            if (!collection)
                throw new CollectionDoesNotExists('Collection does not exists');
            if (this.columnExists(collection, column.name))
                throw new FieldAlreadyExists('Collection already has a field with the same name');
            yield this.axios.post(`v0/meta/bases/${this.baseId}/tables/${collectionName}/addColumn`, { column: this.sqlSchemaTranslator.wixColumnToAirtableColumn(column) });
        });
    }
    removeColumn(collectionName, columnName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, velo_external_db_commons_1.validateSystemFields)(columnName);
            const collection = yield this.describeCollection(collectionName);
            if (!collection)
                throw new CollectionDoesNotExists('Collection does not exists');
            if (!this.columnExists(collection, columnName))
                throw new FieldDoesNotExist('Collection does not contain a field with this name');
            yield this.axios.post(`v0/meta/bases/${this.baseId}/tables/${collectionName}/removeColumn`, { column: columnName });
        });
    }
    describeCollection(collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const collection = (yield this.list()).find(schemas => schemas.id === collectionName);
            if (!collection)
                throw new CollectionDoesNotExists('Collection does not exists');
            return collection.fields;
        });
    }
    translateDbTypes(type) {
        return this.sqlSchemaTranslator.translateType(type);
    }
    extractTableData(data) {
        return data.tables
            .reduce((pV, cV) => (Object.assign(Object.assign({}, pV), { [cV.name]: {
                fields: cV.fields
                    .map((field) => ({ field: field.name, type: this.translateDbTypes(field.type) }))
            } })), {});
    }
    columnExists(fields, columnName) {
        return fields.some((f) => f.field === columnName);
    }
}
exports.default = SchemaProvider;
//# sourceMappingURL=airtable_schema_provider.js.map